@@ -1,12 +1,13 @@
-from pymodbus.exceptions import ParameterException
+from pymodbus.exceptions import ParameterException, NoSuchSlaveException
 from pymodbus.interfaces import IModbusSlaveContext
 from pymodbus.datastore.store import ModbusSequentialDataBlock
 from pymodbus.constants import Defaults
+from pymodbus.compat import iteritems, itervalues
 
 #---------------------------------------------------------------------------#
 # Logging
 #---------------------------------------------------------------------------#
-import logging;
+import logging
 _logger = logging.getLogger(__name__)
 
 
@@ -35,6 +36,7 @@ class ModbusSlaveContext(IModbusSlaveContext):
         self.store['c'] = kwargs.get('co', ModbusSequentialDataBlock.create())
         self.store['i'] = kwargs.get('ir', ModbusSequentialDataBlock.create())
         self.store['h'] = kwargs.get('hr', ModbusSequentialDataBlock.create())
+        self.zero_mode  = kwargs.get('zero_mode', Defaults.ZeroMode)
 
     def __str__(self):
         ''' Returns a string representation of the context
@@ -45,7 +47,7 @@ class ModbusSlaveContext(IModbusSlaveContext):
 
     def reset(self):
         ''' Resets all the datastores to their default values '''
-        for datastore in self.store.values():
+        for datastore in itervalues(self.store):
             datastore.reset()
 
     def validate(self, fx, address, count=1):
@@ -56,7 +58,7 @@ class ModbusSlaveContext(IModbusSlaveContext):
         :param count: The number of values to test
         :returns: True if the request in within range, False otherwise
         '''
-        address = address + 1  # section 4.4 of specification
+        if not self.zero_mode: address = address + 1
         _logger.debug("validate[%d] %d:%d" % (fx, address, count))
         return self.store[self.decode(fx)].validate(address, count)
 
@@ -68,7 +70,7 @@ class ModbusSlaveContext(IModbusSlaveContext):
         :param count: The number of values to retrieve
         :returns: The requested values from a:a+c
         '''
-        address = address + 1  # section 4.4 of specification
+        if not self.zero_mode: address = address + 1
         _logger.debug("getValues[%d] %d:%d" % (fx, address, count))
         return self.store[self.decode(fx)].getValues(address, count)
 
@@ -79,7 +81,7 @@ class ModbusSlaveContext(IModbusSlaveContext):
         :param address: The starting address
         :param values: The new values to be set
         '''
-        address = address + 1  # section 4.4 of specification
+        if not self.zero_mode: address = address + 1
         _logger.debug("setValues[%d] %d:%d" % (fx, address, len(values)))
         self.store[self.decode(fx)].setValues(address, values)
 
@@ -109,7 +111,7 @@ class ModbusServerContext(object):
 
         :returns: An iterator over the slave contexts
         '''
-        return self.__slaves.iteritems()
+        return iteritems(self.__slaves)
 
     def __contains__(self, slave):
         ''' Check if the given slave is in this list
@@ -120,18 +122,29 @@ class ModbusServerContext(object):
         return slave in self.__slaves
 
     def __setitem__(self, slave, context):
-        ''' Wrapper used to access the slave context
+        ''' Used to set a new slave context
 
-        :param slave: slave The context to set
+        :param slave: The slave context to set
         :param context: The new context to set for this slave
         '''
         if self.single: slave = Defaults.UnitId
         if 0xf7 >= slave >= 0x00:
             self.__slaves[slave] = context
-        else: raise ParameterException('slave index out of range')
+        else:
+            raise NoSuchSlaveException('slave index :{} out of range'.format(slave))
+
+    def __delitem__(self, slave):
+        ''' Wrapper used to access the slave context
+
+        :param slave: The slave context to remove
+        '''
+        if not self.single and (0xf7 >= slave >= 0x00):
+            del self.__slaves[slave]
+        else:
+            raise NoSuchSlaveException('slave index: {} out of range'.format(slave))
 
     def __getitem__(self, slave):
-        ''' Wrapper used to access the slave context
+        ''' Used to get access to a slave context
 
         :param slave: The slave context to get
         :returns: The requested slave context
@@ -139,4 +152,5 @@ class ModbusServerContext(object):
         if self.single: slave = Defaults.UnitId
         if slave in self.__slaves:
             return self.__slaves.get(slave)
-        else: raise ParameterException("slave does not exist, or is out of range")
+        else:
+            raise NoSuchSlaveException("slave - {} does not exist, or is out of range".format(slave))
