@@ -6,11 +6,12 @@ These are the device management handlers.  They should be
 maintained in the server context and the various methods
 should be inserted in the correct locations.
 """
-from itertools import izip
 from pymodbus.constants import DeviceInformation
 from pymodbus.interfaces import Singleton
 from pymodbus.utilities import dict_property
+from pymodbus.compat import iteritems, itervalues, izip, int2byte
 
+from collections import OrderedDict
 
 #---------------------------------------------------------------------------#
 # Network Access Control
@@ -84,7 +85,7 @@ class ModbusPlusStatistics(object):
     For more information, see the modbus implementation guide page 87.
     '''
 
-    __data = {
+    __data = OrderedDict({
         'node_type_id'                   : [0x00] * 2, # 00
         'software_version_number'        : [0x00] * 2, # 01
         'network_address'                : [0x00] * 2, # 02
@@ -136,7 +137,7 @@ class ModbusPlusStatistics(object):
         'data_slave_input_path'          : [0x00] * 8, # 42-45
         'program_master_outptu_path'     : [0x00] * 8, # 46-49
         'program_slave_input_path'       : [0x00] * 8, # 50-53
-    }
+    })
 
     def __init__(self):
         '''
@@ -150,7 +151,7 @@ class ModbusPlusStatistics(object):
 
         :returns: An iterator of the modbus plus statistics
         '''
-        return self.__data.iteritems()
+        return iteritems(self.__data)
 
     def reset(self):
         ''' This clears all of the modbus plus statistics
@@ -163,7 +164,7 @@ class ModbusPlusStatistics(object):
 
         :returns: 54 16-bit words representing the status
         '''
-        return self.__data.values()
+        return itervalues(self.__data)
 
     def encode(self):
         ''' Returns a summary of the modbus plus statistics
@@ -171,7 +172,7 @@ class ModbusPlusStatistics(object):
         :returns: 54 16-bit words representing the status
         '''
         total, values = [], sum(self.__data.values(), [])
-        for c in xrange(0, len(values), 2):
+        for c in range(0, len(values), 2):
             total.append((values[c] << 8) | values[c+1])
         return total
 
@@ -215,10 +216,10 @@ class ModbusDeviceIdentification(object):
         Initialize the datastore with the elements you need.
         (note acceptable range is [0x00-0x06,0x80-0xFF] inclusive)
 
-        :param info: A dictionary of {int:string} of values
+        :param information: A dictionary of {int:string} of values
         '''
         if isinstance(info, dict):
-            for key in info.keys():
+            for key in info:
                 if (0x06 >= key >= 0x00) or (0x80 > key > 0x08):
                     self.__data[key] = info[key]
 
@@ -227,14 +228,14 @@ class ModbusDeviceIdentification(object):
 
         :returns: An iterator of the device information
         '''
-        return self.__data.iteritems()
+        return iteritems(self.__data)
 
     def summary(self):
         ''' Return a summary of the main items
 
         :returns: An dictionary of the main items
         '''
-        return dict(zip(self.__names, self.__data.itervalues()))
+        return dict(zip(self.__names, itervalues(self.__data)))
 
     def update(self, value):
         ''' Update the values of this identity
@@ -286,9 +287,9 @@ class DeviceInformationFactory(Singleton):
     '''
 
     __lookup = {
-        DeviceInformation.Basic:    lambda c,r,i: c.__gets(r, range(0x00, 0x03)),
-        DeviceInformation.Regular:  lambda c,r,i: c.__gets(r, range(0x00, 0x08)),
-        DeviceInformation.Extended: lambda c,r,i: c.__gets(r, range(0x80, i)),
+        DeviceInformation.Basic:    lambda c,r,i: c.__gets(r, list(range(0x00, 0x03))),
+        DeviceInformation.Regular:  lambda c,r,i: c.__gets(r, list(range(0x00, 0x08))),
+        DeviceInformation.Extended: lambda c,r,i: c.__gets(r, list(range(0x80, i))),
         DeviceInformation.Specific: lambda c,r,i: c.__get(r, i),
     }
 
@@ -415,7 +416,7 @@ class ModbusCountersHandler(object):
 
         :returns: An iterator of the device counters
         '''
-        return izip(self.__names, self.__data.itervalues())
+        return izip(self.__names, itervalues(self.__data))
 
     def update(self, values):
         ''' Update the values of this identity
@@ -423,7 +424,7 @@ class ModbusCountersHandler(object):
 
         :param values: The value to copy values from
         '''
-        for k, v in values.iteritems():
+        for k, v in iteritems(values):
             v += self.__getattribute__(k)
             self.__setattr__(k, v)
 
@@ -438,7 +439,7 @@ class ModbusCountersHandler(object):
         :returns: A byte with each bit representing each counter
         '''
         count, result = 0x01, 0x00
-        for i in self.__data.values():
+        for i in itervalues(self.__data):
             if i != 0x00: result |= count
             count <<= 1
         return result
@@ -458,7 +459,7 @@ class ModbusCountersHandler(object):
 
 
 #---------------------------------------------------------------------------#
-# Main server controll block
+# Main server control block
 #---------------------------------------------------------------------------#
 class ModbusControlBlock(Singleton):
     '''
@@ -513,7 +514,7 @@ class ModbusControlBlock(Singleton):
         :returns: The encoded events packet
         '''
         events = [event.encode() for event in self.__events]
-        return ''.join(events)
+        return b''.join(events)
 
     def clearEvents(self):
         ''' Clears the current list of events
@@ -570,9 +571,11 @@ class ModbusControlBlock(Singleton):
         :param char: The new serial delimiter character
         '''
         if isinstance(char, str):
+            self.__delimiter = char.encode()
+        if isinstance(char, bytes):
             self.__delimiter = char
         elif isinstance(char, int):
-            self.__delimiter = chr(char)
+            self.__delimiter = int2byte(char)
 
     Delimiter = property(lambda s: s.__delimiter, _setDelimiter)
 
@@ -584,7 +587,7 @@ class ModbusControlBlock(Singleton):
 
         :param mapping: Dictionary of key:value pairs to set
         '''
-        for entry in mapping.iteritems():
+        for entry in iteritems(mapping):
             if entry[0] >= 0 and entry[0] < len(self.__diagnostic):
                 self.__diagnostic[entry[0]] = (entry[1] != 0)
 
@@ -594,9 +597,11 @@ class ModbusControlBlock(Singleton):
         :param bit: The bit to get
         :returns: The current value of the requested bit
         '''
-        if bit >= 0 and bit < len(self.__diagnostic):
-            return self.__diagnostic[bit]
-        return None
+        try:
+            if bit and bit >= 0 and bit < len(self.__diagnostic):
+                return self.__diagnostic[bit]
+        except Exception:
+            return None
 
     def getDiagnosticRegister(self):
         ''' This gets the entire diagnostic register
