@@ -19,7 +19,7 @@ class WriteSingleRegisterRequest(ModbusRequest):
     '''
     function_code = 6
     _rtu_frame_size = 8
-
+    
     def __init__(self, address=None, value=None, **kwargs):
         ''' Initializes a new instance
 
@@ -35,9 +35,12 @@ class WriteSingleRegisterRequest(ModbusRequest):
 
         :returns: The encoded packet
         '''
+        packet = struct.pack('>H', self.address)
         if self.skip_encode:
-            return self.value
-        return struct.pack('>HH', self.address, self.value)
+            packet += self.value
+        else:
+            packet += struct.pack('>H', self.value)
+        return packet
 
     def decode(self, data):
         ''' Decode a write single register packet packet request
@@ -45,7 +48,7 @@ class WriteSingleRegisterRequest(ModbusRequest):
         :param data: The request to decode
         '''
         self.address, self.value = struct.unpack('>HH', data)
-
+    
     def execute(self, context):
         ''' Run a write single register request against a datastore
 
@@ -61,12 +64,19 @@ class WriteSingleRegisterRequest(ModbusRequest):
         values = context.getValues(self.function_code, self.address, 1)
         return WriteSingleRegisterResponse(self.address, values[0])
 
+    def get_response_pdu_size(self):
+        """
+        Func_code (1 byte) + Register Address(2 byte) + Register Value (2 bytes)
+        :return: 
+        """
+        return 1 + 2 + 2
+    
     def __str__(self):
         ''' Returns a string representation of the instance
 
         :returns: A string representation of the instance
         '''
-        return "WriteRegisterRequest %d => %d" % (self.address, self.value)
+        return "WriteRegisterRequest %d" % self.address
 
 
 class WriteSingleRegisterResponse(ModbusResponse):
@@ -123,6 +133,7 @@ class WriteMultipleRegistersRequest(ModbusRequest):
     '''
     function_code = 16
     _rtu_byte_count_pos = 6
+    _pdu_length = 5  #func + adress1 + adress2 + outputQuant1 + outputQuant2
 
     def __init__(self, address=None, values=None, **kwargs):
         ''' Initializes a new instance
@@ -132,9 +143,11 @@ class WriteMultipleRegistersRequest(ModbusRequest):
         '''
         ModbusRequest.__init__(self, **kwargs)
         self.address = address
-        self.values = values or []
-        if not hasattr(values, '__iter__'):
+        if values is None:
+            values = []
+        elif not hasattr(values, '__iter__'):
             values = [values]
+        self.values = values
         self.count = len(self.values)
         self.byte_count = self.count * 2
 
@@ -145,7 +158,7 @@ class WriteMultipleRegistersRequest(ModbusRequest):
         '''
         packet = struct.pack('>HHB', self.address, self.count, self.byte_count)
         if self.skip_encode:
-            return packet + ''.join(self.values)
+            return packet + b''.join(self.values)
         
         for value in self.values:
             packet += struct.pack('>H', value)
@@ -228,10 +241,107 @@ class WriteMultipleRegistersResponse(ModbusResponse):
         params = (self.address, self.count)
         return "WriteMultipleRegisterResponse (%d,%d)" % params
 
+class MaskWriteRegisterRequest(ModbusRequest):
+    '''
+    This function code is used to modify the contents of a specified holding
+    register using a combination of an AND mask, an OR mask, and the
+    register's current contents. The function can be used to set or clear
+    individual bits in the register.
+    '''
+    function_code = 0x16
+    _rtu_frame_size = 10
+
+    def __init__(self, address=0x0000, and_mask=0xffff, or_mask=0x0000,
+                 **kwargs):
+        ''' Initializes a new instance
+
+        :param address: The mask pointer address (0x0000 to 0xffff)
+        :param and_mask: The and bitmask to apply to the register address
+        :param or_mask: The or bitmask to apply to the register address
+        '''
+        ModbusRequest.__init__(self, **kwargs)
+        self.address = address
+        self.and_mask = and_mask
+        self.or_mask = or_mask
+
+    def encode(self):
+        ''' Encodes the request packet
+
+        :returns: The byte encoded packet
+        '''
+        return struct.pack('>HHH', self.address, self.and_mask,
+                           self.or_mask)
+
+    def decode(self, data):
+        ''' Decodes the incoming request
+
+        :param data: The data to decode into the address
+        '''
+        self.address, self.and_mask, self.or_mask = struct.unpack('>HHH',
+                                                                  data)
+
+    def execute(self, context):
+        ''' Run a mask write register request against the store
+
+        :param context: The datastore to request from
+        :returns: The populated response
+        '''
+        if not (0x0000 <= self.and_mask <= 0xffff):
+            return self.doException(merror.IllegalValue)
+        if not (0x0000 <= self.or_mask <= 0xffff):
+            return self.doException(merror.IllegalValue)
+        if not context.validate(self.function_code, self.address, 1):
+            return self.doException(merror.IllegalAddress)
+        values = context.getValues(self.function_code, self.address, 1)[0]
+        values = ((values & self.and_mask) | self.or_mask)
+        context.setValues(self.function_code, self.address, [values])
+        return MaskWriteRegisterResponse(self.address, self.and_mask,
+                                         self.or_mask)
+
+
+class MaskWriteRegisterResponse(ModbusResponse):
+    '''
+    The normal response is an echo of the request. The response is returned
+    after the register has been written.
+    '''
+    function_code = 0x16
+    _rtu_frame_size = 10
+
+    def __init__(self, address=0x0000, and_mask=0xffff, or_mask=0x0000,
+                 **kwargs):
+        ''' Initializes a new instance
+
+        :param address: The mask pointer address (0x0000 to 0xffff)
+        :param and_mask: The and bitmask applied to the register address
+        :param or_mask: The or bitmask applied to the register address
+        '''
+        ModbusResponse.__init__(self, **kwargs)
+        self.address = address
+        self.and_mask = and_mask
+        self.or_mask = or_mask
+
+    def encode(self):
+        ''' Encodes the response
+
+        :returns: The byte encoded message
+        '''
+        return struct.pack('>HHH', self.address, self.and_mask,
+                           self.or_mask)
+
+    def decode(self, data):
+        ''' Decodes a the response
+
+        :param data: The packet data to decode
+        '''
+        self.address, self.and_mask, self.or_mask = struct.unpack('>HHH',
+                                                                  data)
+
+
 #---------------------------------------------------------------------------#
 # Exported symbols
 #---------------------------------------------------------------------------#
 __all__ = [
     "WriteSingleRegisterRequest", "WriteSingleRegisterResponse",
     "WriteMultipleRegistersRequest", "WriteMultipleRegistersResponse",
+    "MaskWriteRegisterRequest", "MaskWriteRegisterResponse"
 ]
